<?php

namespace Drupal\taxonomy_proportion\Plugin\Field\FieldFormatter;

use Drupal\Core\Field\Plugin\Field\FieldFormatter\EntityReferenceLabelFormatter;
use Drupal\Core\Field\FieldItemListInterface;
use Drupal\Core\Field\Attribute\FieldFormatter;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\StringTranslation\TranslatableMarkup;

/**
 * Plugin implementation of the 'taxonomy_proportion' formatter.
 */
#[FieldFormatter(
  id: 'taxonomy_proportion_formatter',
  label: new TranslatableMarkup('Taxonomy proportion formatter'),
  description: new TranslatableMarkup('A formatter for Taxonomy proportion'),
  field_types: ['taxonomy_proportion'],
)]
class TaxonomyProportionFormatter extends EntityReferenceLabelFormatter {

  /**
   * {@inheritdoc}
   */
  public static function defaultSettings() {
    return [
      'proportion_first' => FALSE,
    ] + parent::defaultSettings();
  }

  /**
   * {@inheritdoc}
   */
  public function settingsForm(array $form, FormStateInterface $form_state) {
    $elements = parent::settingsForm($form, $form_state);

    $elements['proportion_first'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Display proportion before term'),
      '#default_value' => $this->getSetting('proportion_first'),
    ];

    return $elements;
  }

  /**
   * {@inheritdoc}
   */
  public function viewElements(FieldItemListInterface $items, $langcode) {
    $elements = parent::viewElements($items, $langcode);

    $prefix = $this->getFieldSetting('field_prefix') ?? '';
    $suffix = $this->getFieldSetting('field_suffix') ?? '';
    $proportion_first = $this->getSetting('proportion_first');

    foreach ($elements as $delta => &$element) {
      $proportion_markup = [
        '#theme' => 'taxonomy_proportion',
        '#proportion' => $items[$delta]->proportion,
        '#proportion_prefix' => $prefix,
        '#proportion_suffix' => $suffix,
        '#proportion_first' => $proportion_first,
        '#term' => $element,
      ];

      $element = $proportion_markup;
    }

    return $elements;
  }

  /**
   * {@inheritdoc}
   */
  public function settingsSummary() {
    $summary = parent::settingsSummary();

    // Add information about display order
    $summary[] = $this->t('Display proportion @order', [
      '@order' => $this->getSetting('proportion_first') ? $this->t('before term') : $this->t('after term'),
    ]);

    return $summary;
  }
}
