<?php

namespace Drupal\taxonomy_proportion\Plugin\Field\FieldType;

use Drupal\Core\Field\Attribute\FieldType;
use Drupal\Core\Field\Plugin\Field\FieldType\EntityReferenceItem;
use Drupal\Core\StringTranslation\TranslatableMarkup;
use Drupal\Core\Field\FieldStorageDefinitionInterface;
use Drupal\Core\TypedData\DataDefinition;
use \Drupal\Core\Field\EntityReferenceFieldItemList;
use Drupal\Core\Form\FormStateInterface;

/**
 * Plugin implementation of the 'taxonomy_proportion' field type.
 */
#[FieldType(
  id: "taxonomy_proportion",
  label: new TranslatableMarkup("Taxonomy proportion"),
  description: new TranslatableMarkup("Add a proportion to taxonomy terms."),
  default_widget: "taxonomy_proportion_autocomplete_widget",
  default_formatter: "taxonomy_proportion_formatter",
  list_class: EntityReferenceFieldItemList::class,
  category: "reference",
)]
class TaxonomyProportion extends EntityReferenceItem {

  /**
   * {@inheritdoc}
   */
  public static function defaultFieldSettings() {
    return [
      'field_prefix' => '',
      'field_suffix' => '',
    ] + parent::defaultFieldSettings();
  }

  /**
   * {@inheritdoc}
   */
  public static function propertyDefinitions(FieldStorageDefinitionInterface $field_definition) {
    // Get the properties of the entity reference
    $properties = parent::propertyDefinitions($field_definition);

    // Add the proportion property
    $properties['proportion'] = DataDefinition::create('float')
      ->setLabel(t('Proportion'))
      ->setRequired(TRUE)
      ->setDescription(t('The proportion value associated with the taxonomy term.'));

    return $properties;
  }

  /**
   * {@inheritdoc}
   */
  public static function schema(FieldStorageDefinitionInterface $field_definition) {
    // Get the schema of the entity reference
    $schema = parent::schema($field_definition);

    // Add the proportion column
    $schema['columns']['proportion'] = [
      'type' => 'float',
      'not null' => TRUE,
    ];

    return $schema;
  }

  /**
   * {@inheritdoc}
   */
  public function fieldSettingsForm(array $form, FormStateInterface $form_state) {
    $element = parent::fieldSettingsForm($form, $form_state);

    $element['field_prefix'] = [
      '#type' => 'textfield',
      '#title' => t('Prefix'),
      '#default_value' => $this->getSetting('field_prefix'),
      '#description' => t("Define a string that should be prefixed to the value, like '$ ' or '€ '. Leave blank for none."),
    ];

    $element['field_suffix'] = [
      '#type' => 'textfield',
      '#title' => t('Suffix'),
      '#default_value' => $this->getSetting('field_suffix'),
      '#description' => t("Define a string that should be suffixed to the value, like ' %'. Leave blank for none."),
    ];

    return $element;
  }

  /**
   * {@inheritdoc}
   */
  public static function getPreconfiguredOptions() {
    return [];
  }

  /**
   * {@inheritdoc}
   */
  public static function defaultStorageSettings() {
    return [
      'target_type' => 'taxonomy_term',
    ] + parent::defaultStorageSettings();
  }

  /**
   * {@inheritdoc}
   */
  public function storageSettingsForm(array &$form, FormStateInterface $form_state, $has_data) {
    return [];
  }
}
