<?php

namespace Drupal\taxonomy_proportion\Plugin\Field\FieldWidget;

use Drupal\Core\Field\Plugin\Field\FieldWidget\EntityReferenceAutocompleteWidget;
use Drupal\Core\Field\Attribute\FieldWidget;
use Drupal\Core\Field\FieldItemListInterface;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\StringTranslation\TranslatableMarkup;

/**
 * Plugin implementation of the 'taxonomy_proportion_autoccomplete' widget.
 */
#[FieldWidget(
  id: 'taxonomy_proportion_autocomplete_widget',
  label: new TranslatableMarkup('Autocomplete term'),
  description: new TranslatableMarkup('An autocomplete text field with proportion.'),
  field_types: ['taxonomy_proportion'],
)]
class TaxonomyProportionAutocompleteWidget extends EntityReferenceAutocompleteWidget {

  /**
   * {@inheritdoc}
   */
  public static function defaultSettings() {
    return [
      'proportion_min_value' => 0,
      'proportion_max_value' => 100,
    ] + parent::defaultSettings();
  }

  /**
   * {@inheritdoc}
   */
  public function formElement(FieldItemListInterface $items, $delta, array $element, array &$form, FormStateInterface $form_state) {
    $element = parent::formElement($items, $delta, $element, $form, $form_state);
    $item = $items[$delta];

    $element['target_id']['#weight'] = 0;

    $element['proportion'] = [
      '#type' => 'number',
      '#title' => $this->t('Proportion'),
      '#default_value' => $item->proportion,
      '#min' => $this->getSetting('proportion_min_value'),
      '#max' => $this->getSetting('proportion_max_value'),
      '#step' => 0.1,
      '#weight' => 1,
      '#required' => FALSE,
    ];

    return $element;
  }
}