<?php

namespace Drupal\taxonomy_proportion\Plugin\Field\FieldWidget;

use Drupal\Core\Field\FieldItemListInterface;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Field\Plugin\Field\FieldWidget\OptionsWidgetBase;
use Drupal\Core\Field\Attribute\FieldWidget;
use Drupal\Core\StringTranslation\TranslatableMarkup;

/**
 * Plugin implementation of the 'taxonomy_proportion_checkbox' widget.
 */
#[FieldWidget(
  id: 'taxonomy_proportion_checkbox_widget',
  label: new TranslatableMarkup('Checkboxes with proportion'),
  description: new TranslatableMarkup('Checkboxes with proportion values.'),
  field_types: ['taxonomy_proportion'],
  multiple_values: TRUE
)]
class TaxonomyProportionCheckboxWidget extends OptionsWidgetBase {

  /**
   * {@inheritdoc}
   */
  public static function defaultSettings() {
    return [
      'proportion_min_value' => 0,
      'proportion_max_value' => 100,
    ] + parent::defaultSettings();
  }

  /**
   * {@inheritdoc}
   */
  public function formElement(FieldItemListInterface $items, $delta, array $element, array &$form, FormStateInterface $form_state) {
    $options = $this->getOptions($items->getEntity());
    $selected = $this->getSelectedOptions($items);

    // Add parent fieldset with vocabulary title
    $element += [
      '#type' => 'fieldset',
      '#title' => $this->fieldDefinition->getLabel(),
    ];

    foreach ($options as $key => $choice) {
      $element[$key]['target_id'] = [
        '#type' => 'checkbox',
        '#title' => $choice,
        '#default_value' => in_array($key, $selected),
        '#return_value' => $key,
      ];

      $element[$key]['proportion'] = [
        '#type' => 'number',
        '#title' => $this->t('Proportion'),
        '#default_value' => $this->getOptionProportion($items, $key),
        '#min' => $this->getSetting('proportion_min_value'),
        '#max' => $this->getSetting('proportion_max_value'),
        '#step' => 0.1,
      ];
    }

    return $element;
  }

  /**
   * Helper to get the proportion of an option.
   */
  protected function getOptionProportion(FieldItemListInterface $items, $key) {
    foreach ($items as $item) {
      if ($item->target_id == $key) {
        return $item->proportion;
      }
    }
    return NULL;
  }

  /**
   * {@inheritdoc}
   */
  public function massageFormValues(array $values, array $form, FormStateInterface $form_state) {
    foreach ($values as $target_id => $value) {
      if (empty($value['target_id'])) {
        unset($values[$target_id]);
      }
    }
    return $values;
  }
}
